#!/usr/bin/env apl --script

⍝ © 1981-1983 2023-2024 LAAS CNRS, Toulouse, France
⍝ Use of this software is governed by the MIT licence
⍝ Author Bernard Berthomieu, LAAS CNRS, Toulouse, France

∇B←SETMULTI;V
 B←1
 →(L∆SCRIPTING)/CHECK
 ⍝ 2023 addition: SETS L∆MULTI
 V←'MULTI-ENABLEDNESS OPTION IS ',(⍕L∆MULTI),'; ENTER NEW VALUE (0 OR 1) OR RETURN: '
 V←(⍴V)↓⍞,0/⍞←V
 B←1
 →(0=⍴V)/CHECK             ⍝ no change
 L∆MULTI←1↑⍎V              ⍝ update multi (remember: must convert string V to an int !!)
CHECK: →(0=L∆MULTI)/0      ⍝ multi not set; no check needed
 ⍝ multi set; B=1
 →(∧/[1]∨/[2]es[1;;])/0  ⍝ no tr is w-enabled
 'SOME TRANSITION IS ⍵-ENABLED; TERMINATING'
 B←0
 →0
∇

∇SETSHIFT;V
 ⍝ 2023 addition: SETS L∆SHIFT
 V←'SHIFT OPTION IS ',(⍕L∆SHIFT),'; ENTER NEW VALUE (0 OR 1) OR RETURN: '
 V←(⍴V)↓⍞,0/⍞←V
 →(0=⍴V)/0                 ⍝ no change
 L∆SHIFT←1↑⍎V              ⍝ update
∇

∇SETUP
 ⍝ 2023 addition
 SYS∆NETS←0⍴''
 SYS∆VARSNETS←0 0 ⍴''
 L∆CPU←300
 L∆CLASSES←1000
 L∆STATES←0⍴''
 CLEARCONTEXT
 CLEARNET
 SYS∆VARSANA←0 0 ⍴''  ⍝ what for ?
 L∆BOUNDEDNESSTEST←0
 ⍝ L∆BOUNDEDTESTLEVEL←0
 L∆CONDITIONS←''
 L∆PRINT←60
 L∆MULTI←0
 L∆SHIFT←0
 ⍝ FOR SCRIPTING
 L∆SCRIPTING←0
 L∆VERBOSE←0
 L∆LIVE←1
∇

∇TOLEGACYNET NAME;I;N
 ⍝ 2023 -- creates legacy definition of net loaded, with name NAME
 ⍝ useful to convert exec models to legacy models, e.g.
 ⍝ trains3e ⋄ TOLEGACYNET 'TRAINS3E'
 N←NAME
 I←1
 DEB:→(I>(⍴tn)[1])/END
 N←N CONCAT1 '''',(TRANSDEF I),''''
 I←I+1
 →DEB
 END: N←N CONCAT1 '''',M0DEF,''''
 ⎕FX N
 'NET ',NAME,' CREATED'
∇

∇TOEXECNET NAME;tl;tw;pl;pw;c;s;H;T;P;M;I;E;N;B
 ⍝ 2023 -- creates exec definition of net loaded, with name NAME
 ⍝ useful to convert exec models to legacy models, e.g.
 ⍝ SETUP ⋄ READNET 'ZUBEK' ⋄ TOEXECNET 'zubek'
 ⍝ header
 H←NAME
 ⍝ transitions
 tl←(⍴tn)[1]
 tw←(⍴tn)[2]
 c←((tl,1)⍴'"')
 s←((tl,1)⍴' ')
 T ← 'tn ← 1↓[1]⊃"" ',,(tl,3+tw)⍴,c,[2]tn,[2]c,[2]s   ⍝ tn must be a matrix
 ⍝ places
 pl←(⍴pn)[1]
 pw←(⍴pn)[2]
 c←((pl,1)⍴'"')
 s←((pl,1)⍴' ')
 P ← 'pn ← 1↓[1]⊃"" ',,(pl,3+pw)⍴,c,[2]pn,[2]c,[2]s   ⍝ pn must be a matrix
 ⍝ initial marking
 M ← 'm0 ← ',(⍴m0),'⍴',,m0
 ⍝ static interval bounds
 I ← 'is ← ',(2,tl,1),'⍴',,is
 ⍝ input, output matrices
 E ← 'es ← ',(2,tl,pl),'⍴',,es
 ⍝ net name
 N ← 'nn← ','"',NAME,'"'
 B ← (⍕H) (⍕T) (⍕P) (⍕M) (⍕I) (⍕E) (⍕N)
 ⎕FX B
 'NET ',NAME,' CREATED'
 ∇

∇F←FORMAT NOM
 ⍝ returns format of net (1 if exec, 0 otherwise)
 F←∨/,(1 0 ↓⎕CR NOM) ∊ '←'
∇

∇tina_s N
  ⍝ 2023 addition -- script version
  →(FORMAT ⊃N)/EXC
  ⍝ legacy format
  CLEARNET
  READNET N
  →BDNESS
EXC: ⍝ exec format
  ⍎N
  LOADED
BDNESS:
  →(0=SETMULTI)/DONE   ⍝ w-enabled
  BOUNDED
  →(0=L∆VERBOSE)/LVNESS
  ⍝ verbose
  PRINTCLASSES
  ' '
  PRINTGRAPH
  ' '
LVNESS: →(0=L∆LIVE)/DONE
  LIVE
  →(0=L∆VERBOSE)/DONE
  LIVEDIAGNOSIS
  PRINTCFCMS
  ' '
DONE: 'DONE'
  ⍎')off'
∇

∇tina N
  SETUP
  ⍝ 2023 addition -- interactive version
  →(FORMAT ⊃N)/EXC
  ⍝ legacy format
  CLEARNET
  READNET N
  →BDNESS
EXC: ⍝ exec format
  ⍎N
  LOADED
BDNESS:
  OPTIONS
  SETSHIFT
  →(0=SETMULTI)/DONE   ⍝ w-enabled
  BOUNDED
  ⍝ asks user confirmation for those:
  V←'PRINT CLASSES ? ENTER ANY CHAR FOR YES OR RETURN: '
  V←(⍴V)↓⍞,0/⍞←V
  →(0=⍴,V)/GRAPH
  PRINTCLASSES
  ' '
GRAPH:
  V←'PRINT CLASS GRAPH ? ENTER ANY CHAR FOR YES OR RETURN: '
  V←(⍴V)↓⍞,0/⍞←V
  →(0=⍴,V)/LVNESS
  PRINTGRAPH
  ' '
LVNESS:
  V←'COMPUTE LIVENESS ANALYSIS ? ENTER ANY CHAR FOR YES OR RETURN: '
  V←(⍴V)↓⍞,0/⍞←V
  →(0=⍴,V)/DONE
  LIVE
  V←'PRINT LIVENESS DETAILS ? ENTER ANY CHAR FOR YES OR RETURN: '
  V←(⍴V)↓⍞,0/⍞←V
  →(0=⍴,V)/DONE
  LIVEDIAGNOSIS
  PRINTCFCMS
  ' '
DONE: 'DONE'
∇


